#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
# This file has been generated on [2021-05-12 08:46:29]
# Using platform [Windows fv-az122-648 Intel64 Family 6 Model 63 Stepping 2, GenuineIntel]

DATE='2021-05-12 08:46:29'
PLATFORM='Windows fv-az122-648 Intel64 Family 6 Model 63 Stepping 2, GenuineIntel'
PFX='#'

HAS_HDF5=True
HAS_H5PY=0
HAS_PY2=0
HAS_PY3=1
HAS_VTK=True
HAS_MSW=1
USE_COMPACT_STORAGE=1

INCLUDE_DIRS=['D:\\bld\\pycgns_1620808498069\\_h_env\\lib\\site-packages\\numpy\\core\\include', 'D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\include', 'D:\\bld\\pycgns_1620808498069\\_h_env/lib/python3.9/site-packages/numpy/core/include', '', 'D:\\bld\\pycgns_1620808498069\\_h_env/include', '../include', '/include']
LIBRARY_DIRS=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\lib', '', 'D:\\bld\\pycgns_1620808498069\\_h_env/lib', '../lib', '/lib64', '/lib']

HDF5_VERSION='1.10.6'
HDF5_PATH_INCLUDES=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\include']
HDF5_PATH_LIBRARIES=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\lib']
HDF5_LINK_LIBRARIES=['hdf5']
HDF5_EXTRA_ARGS=[]

NUMPY_VERSION='1.19.5'
NUMPY_VERSION_API='0x01000009'
NUMPY_PATH_INCLUDES=['D:\\bld\\pycgns_1620808498069\\_h_env\\lib\\site-packages\\numpy\\core\\include']
NUMPY_PATH_LIBRARIES=['D:\\bld\\pycgns_1620808498069\\_h_env\\Library\\lib']
NUMPY_LINK_LIBRARIES=[]
NUMPY_EXTRA_ARGS=[]

PYQT_VERSION='1.9.0'
QT_VERSION='5.12.9'
CYTHON_VERSION='0.29.23'
VTK_VERSION='9.0.1'

__version__=5 # @@UPDATEVERSION@@
__release__=3 # @@UPDATERELEASE@@
__revision__=1 # @@UPDATEREVISION@@
__vid__="%s.%s.%s"%(__version__,__release__,__revision__)
__doc__='pyCGNS - %s - Python package for CFD General Notation System'        %(__vid__)
version=__vid__

NAME='pyCGNS'
VERSION=__vid__
DESCRIPTION=__doc__
AUTHOR='Marc Poinot'
EMAIL="marc.poinot@safrangroup.com"
LICENSE="LGPL 2"

