#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
"""
  pyCGNS - Python package for CFD General Notation System

  This packages provides some libs and tools dedicated to the CGNS standard.

  Modules are:

    * CGNS.MAP
    * CGNS.APP
    * CGNS.NAV
    * CGNS.DAT
    * CGNS.VAL
    * CGNS.PAT
    
"""
#
from CGNS.pyCGNSconfig import version as __version__
#
# --- last line
