#  -------------------------------------------------------------------------
#  pyCGNS - Python package for CFD General Notation System -
#  See license.txt file in the root directory of this Python module source  
#  -------------------------------------------------------------------------
#
# actual versions number are in setuputils.py
#
from CGNS import pyCGNSconfig
__version__ = pyCGNSconfig.__version__
__release__ = pyCGNSconfig.__release__
__vid__ = pyCGNSconfig.__vid__
__doc__ = pyCGNSconfig.__doc__
version = __vid__
id = __vid__
#
# --- last line
